/*

 Habillage d'une courbe ayant pour section une courbe plane (une ellipse dans le programme)  
 

Programme crit et mis en ligne sur Internet par Alain Esculier en juillet 2022 


*/

#version 3.8;
global_settings { assumed_gamma 2. }  


#include "colors.inc"
#include "metals.inc" 
#include "glass.inc"
#include "stones.inc"  

    
camera  {orthographic  location <-0,5,-30> look_at<0,0,0> right 4/3*x    }  // rotate myclock*y }
light_source { <-10,30,-30> White }  // shadowless 
light_source { <10,30,-30>  White  shadowless}  //color <1,0.98,0.9>  // 
light_source { <4,-20,-30>  White   shadowless}   // color <1,0.95,0>*0.3 
background{     White}   //   Black}   //  


//=================== INITIALISATIONS DE LA COURBE ET DU TRIEDRE MOBILE =====================  
#declare numCourbe=3;             //--- entre 1 et 4 -----
 
#declare coeff=1.5; #declare a=0.6*coeff; #declare b=0.2*coeff; //--- demi-diamtres de la section elliptique
#declare ro=0.02*coeff;  //--- rayon des courbes
#declare R=2; 
#declare V=<0,0,1>;               //--- B orthogonal au plan (T,V) ; T vecteur tangent -----
#declare Serret_Frenet=false;    //--- choix tridre

//=================================================================
#switch(numCourbe)
      #case(1)
          //------------------- Noeud de trfle normalL ---------------------------- 
          #macro ML(uu)  <cos(uu)+2*cos(2*uu), sin(uu)-2*sin(2*uu),2*sin(3*uu)>    #end    
          #macro dMLu(uu)    <-sin(uu)-4*sin(2*uu), cos(uu)-4*cos(2*uu), 6*cos(3*uu)>     #end   
          #macro dML2u(uu)   <-cos(uu)-8*cos(2*uu), -sin(uu)+8*sin(2*uu), -18*sin(3*uu)> #end 
          #declare echelle=0.15;  
          #break
      #case(2)
          //------------------- TORE TORDU ---------------------------- 
          #declare R=2; 
          #macro ML(uu)      < R*cos(uu), R*sin(uu), R*sin(2*uu)> #end      
          #macro dMLu(uu)    < -R*sin(uu), R*cos(uu),2*R*cos(2*uu)> #end   
          #macro dML2u(uu)   < -R*cos(uu), -R*sin(uu), -4*R*sin(2*uu)> #end   
          #declare echelle=0.15;  
          #break
      #case(3)
          //------------------- NOEUD TORIQUE ---------------------------- 
              #declare p=2; #declare q=3;
              #declare R=6; #declare r=1.5;    
          #macro ML(uu)                   < (R+r*cos(q*uu))*cos(p*uu), (R+r*cos(q*uu))*sin(p*uu), r*sin(q*uu)>  #end//torique   p = nbre de tours q= nbre d'oscillations  
          #macro dMLu(uu)     #declare p=2; #declare q=3;
                        <-r*sin(q*uu)*q*cos(p*uu)-(R+r*cos(q*uu))*sin(p*uu)*p, -r*sin(q*uu)*q*sin(p*uu)+(R+r*cos(q*uu))*cos(p*uu)*p, r*cos(q*uu)*q >     #end   //torique
          #macro dML2u(uu)   // <0,0,1>    #end //rot(dMLu(uu),uu/NF,<0,0,1>)   #end //  //torique
                  #declare p=2; #declare q=3;
                   < -r*cos(q*uu)*q*q*cos(p*uu)+2*r*sin(q*uu)*q*sin(p*uu)*p-(R+r*cos(q*uu))*cos(p*uu)*p*p, -r*cos(q*uu)*q*q*sin(p*uu)-2*r*sin(q*uu)*q*cos(p*uu)*p-(R+r*cos(q*uu))*sin(p*uu)*p*p, -r*sin(q*uu)*q*q  >  #end  //torique
          #declare echelle=0.07;  
          #break
      #case(4)
         //------------------- TORE SIMPLE ---------------------------- 
          #macro ML(uu)      < R*cos(uu), R*sin(uu), 0> #end   
          #macro dMLu(uu)      < -R*sin(uu), R*cos(uu),0> #end  
          #macro dML2u(uu)     < -R*cos(uu), -R*sin(uu),0> #end   
          #declare echelle=0.2;  
          #break
      #break
#end
/**/
 
// ----------------------------------------
#macro norm(vec) vec/sqrt(vdot(vec,vec)) #end  

//==============  repre mobile li au point courant ===================
// ----------------------------------------
#macro T(uu) norm(dMLu(uu)) #end    //---vecteur tangent  la courbe directrice

// ---------------------- choix le vecteur  B(uu) ------------------
#switch(Serret_Frenet)
     #case(true)   //----- utilisation Serret-Frenet 
           #macro B(uu) norm(vcross(dMLu(uu),dML2u(uu))) #end  #macro N(uu) vcross(B(uu),T(uu)) #end 
           #break
     #case(false)   //----- utilisation du vecteur V
           //------ souvent moins tordu qu'avec le prcdent ;(on peut aussi changer <0,0,1>selon la courbe !) ------
           #declare V= <0,0,1> ;            
           #macro B(uu) norm(vcross(dMLu(uu),V)) #end  
           #macro N(uu) vcross(B(uu),T(uu)) #end     
           #break
     #break
 #end 
//======================================================================
/*           
*/
  
// ======== SECTION ELLIPTIQUE    ================

#macro section(uu,tt)   ML(uu)+a*cos(tt)*N(uu) + b*sin(tt)*B(uu) #end   




//=========================================== 

#declare traceSection=
    union{  
        #declare nsec=50; #declare pas=2*pi/nsec;   //---- changer selon dsir
        #declare npasSection=200;    #declare pasSection=2*pi/npasSection; 
        #for(i,0,nsec) 
               #for(j,0,npasSection)    
                    cylinder{ section(i*pas,j*pasSection), section(i*pas,(j+1)*pasSection),ro  } 
                    sphere{ section(i*pas, j*pasSection),ro}
                #end 
        #end        
        pigment {Blue}
       };   
/**/
#declare traceCourbe=
    union{  
        #declare nsec=200; #declare pas=2*pi/nsec;   //---- changer selon dsir
        #for(i,0,nsec-1) 
              cylinder{ ML(i*pas), ML((i+1)*pas),2*ro  } 
              sphere{ ML(i*pas),ro}
              
        #end        
        pigment {Magenta}
       };   
#declare traceCourbesHabillage=
    union{ 
        #macro pt(uu,vv) section(vv,uu) #end   
        #declare nsec=200; #declare passec=2*pi/nsec;   //---- sur la section  
        #declare npara=10; #declare paspara=2*pi/npara;  //---- changer selon dsir
        #for(i,0,npara-1) 
            #for(j,0,nsec-1)
              cylinder{ section(j*passec,i*paspara),section((j+1)*passec,i*paspara),ro  } 
              sphere{  section(j*passec,i*paspara),2*ro}
            #end  
        #end        
        pigment {Gold}
       }; 

//================== macro =============      

#macro surf(umin,umax,vmin,vmax,uiter,viter)      
       #macro pt(uu,vv) section(vv,uu) #end   
       union{   
        #local iu=(umax-umin)/uiter; 
        #local iv=(vmax-vmin)/viter;
       
        #for(m,0,uiter-1,1)             //boucle sur uu
             #local uu = umin+m*iu;                   
             #for(n,0,viter-1,1)        //boucle sur vv  
                  #local vv = vmin+n*iv;
                   #local p1=pt(uu,vv);   
                  #local p2=pt(uu+iu,vv);   
                  #local p3=pt(uu+iu,vv+iv);    
                  #local p4=pt(uu,vv+iv);
                 //Calcul des normales
                  #local cf=2;                
                  #local uu=uu+cf*iu ; #local k1=pt(uu,vv);  
                  #local uu=uu-cf*iu ; #local vv=vv+cf*iv; #local k2=pt(uu,vv);  
                  #local vv=vv-cf*iv ; #local uu=uu-cf/2*iu ;  #local k3=pt(uu,vv);
                  #local uu=uu+cf/2*iu;    #local vv=vv-cf/2*iv ;  #local k4=pt(uu,vv);
                  //#local vv=vv+cf/2*iv; 
                  #local n1=vcross(k2-p4,k1-p2);  #local n2=vcross(p4-p3,k2-p4);    #local n3=vcross(k3-p1,k2-p4);   
                  #local n4=vcross(p3-p2,k1-p2);  #local n5=vcross(p1-p2,p3-p2);    #local n6=vcross(k3-p1,p4-p1) ;
                  #local n7=vcross(k1-p2,k4-p1);   #local n8=vcross(k4-p1,p1-p2) ;  #local n9=vcross(k4-p1,k3-p1);
                 
                  //TRIANGLES 
                   object{  smooth_triangle{   p1,n5+n6+n8+n9,        p2,n4+n5+n7+n8,       p3,n1+n2+n4+n5   }  }
                  object{  smooth_triangle{   p1, n5+n6+n8+n9,       p3, n1+n2+n4+n5       p4, n2+n3+n5+n6  }  } 
                  
                  //  #local vv = vv+iv;
                 #end                 // fin boucle vv
             #end                   //fin boucle uu  
             } //---  fin union
       #end     //--- fin de la macro surf  
 //=============  TRACS   ====================  
union{  
    //  object{ traceCourbe } 
   //     object{ traceSection } 
 //   object{ traceCourbesHabillage }
    object{ surf(0,2*pi,0,2*pi,300,100)  texture{T_Stone26 scale 3} }  // pigment{Cyan}  }
        
     rotate <50,-30,0>        
      scale echelle  
      }
      
       
       /*
//====================================================================
// ======== SECTION POLYGONALE============================= 
#declare NP=3; // nombre de points  et coordonnes dans le repre N(uu),B(uu)  
#declare coef=0.5;
#declare polygone=array[NP]; 
#for(i,0,NP-1) #declare polygone[i]=<cos(i*2*pi/NP),sin(i*2*pi/NP)> ;#end  
   
#macro section(pas,uu) 
       #local TP1=array[NP];     #for(i,0,NP-1)  #local TP1[i]=ML(uu)+polygone[i].x*N(uu)+polygone[i].y*B(uu); #end 
       #local TP2=array[NP];     #for(i,0,NP-1)  #local TP2[i]=ML(uu+pas)+polygone[i].x*N(uu+pas)+polygone[i].y*B(uu+pas); #end  
       union{ #for(i,0,NP-1)  
                             triangle{ TP1[i], TP1[mod(i+1,NP)], TP2[mod(i+1,NP)]     }
                             triangle{ TP2[i], TP2[mod(i+1,NP)], TP1[i] } 
              #end   }
#end 
  
//===========================================
 #declare ro=0.04;    
#declare aretes= union{ 
    #declare na=300;  #declare pasa=2*pi/na;
     #for(i,0,NP-1)
           #for(j,0,na) cylinder{ML(j*pasa)+polygone[i].x*N(j*pasa)+polygone[i].y*B(j*pasa),  
                                 ML((j+1)*pasa)+polygone[i].x*N((j+1)*pasa)+polygone[i].y*B((j+1)*pasa),ro}  
                        sphere{ML(j*pasa)+polygone[i].x*N(j*pasa)+polygone[i].y*B(j*pasa),ro}  
           #end 
     #end
           pigment{Gold}
           };
//===========================================
#declare Npas=800; 
#declare pas=2*pi/Npas;
#declare habillage=union{ #for(j,0, Npas) section(pas,j*pas) #end
          pigment{Cyan transmit 0.6}
           }   
           
           
#declare courbe=union{
                   #declare nc=300;    #declare pasC=2*pi/nc;
                   #for(i,0,nc) cylinder{ML(i*pasC),ML((i+1)*pasC), 0.04} sphere {ML(i*pasC), 0.04}  #end
                    pigment{Turquoise}
           
            }
//===========================================
#declare Les_sections= union{  
        #declare nsec=50; #declare pas=2*pi/nsec;  #declare ro=0.04; //---- changer selon dsir
       
        #for(j,0,nsec) 
                   
                    polygon{NP, #for(i,0,NP-1) ML(j*pas)+polygone[i].x*N(j*pas)+polygone[i].y*B(j*pas) #end  pigment {Blue}  }
                                
                   #for(i,0,NP-1)   #if(i=NP-1) #declare k=0; #else  #declare k=i+1; #end 
                    cylinder{  ML(j*pas)+polygone[i].x*N(j*pas)+polygone[i].y*B(j*pas) 
                              ,ML(j*pas)+polygone[k].x*N(j*pas)+polygone[k].y*B(j*pas),ro  pigment {Red} }   
                       sphere {  ML(j*pas)+polygone[i].x*N(j*pas)+polygone[i].y*B(j*pas)  ,1.2*ro  pigment {Red} }                    
                                         
                    #end 
        #end  
               
               };   
//=============================
union{ object{courbe}
       object{habillage}
       object{Les_sections}
        object{aretes} 
       rotate <60,-20,0>   
       translate 1.*y
       scale 0.7
       }

*/ 
 
 
 
 
